SystemOrganization addCategory: #'PostMark-Core'!

Error subclass: #PMException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostMark-Core'!

!PMException commentStamp: 'FrancoisStephany 12/15/2011 19:01' prior: 0!
I'm thrown when an error is encountered within PostMark.
See http://developer.postmarkapp.com/developer-build.html for more details about the errors that could occur during a transaction.!

Object subclass: #PMInterface
	instanceVariableNames: 'apiKey apiUrl debugStream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostMark-Core'!

!PMInterface commentStamp: 'FrancoisStephany 1/11/2012 00:16' prior: 0!
| file email interface |

file := (FSLocator imageDirectory / 'image.jpg') resolve.
	
email := PMEmail new
	from: 'francois@wapict.be'
	to: 'francois@yopmail.com';
	subject: 'Test Application';
	textBody: 'Yeah baby';
	addAttachment: 'image.jpg' content: stream readStream contents contentType: 'image/jpeg';
	yourself.

interface := PMInterface new.
interface apiKey: 'blopblop'.

interface send: email.!

!PMInterface methodsFor: 'accessing' stamp: 'FrancoisStephany 12/15/2011 17:38'!
apiKey
	^ apiKey ifNil: [ apiKey := self defaultApiKey ]! !

!PMInterface methodsFor: 'accessing' stamp: 'FrancoisStephany 12/15/2011 17:22'!
apiKey: aByteString 
	apiKey := aByteString! !

!PMInterface methodsFor: 'accessing' stamp: 'FrancoisStephany 12/15/2011 17:39'!
apiUrl
	^apiUrl ifNil: [ apiUrl  := self defaultApiUrl ].! !

!PMInterface methodsFor: 'private' stamp: 'FrancoisStephany 12/15/2011 18:12'!
buildZnClientForContent: aString
	| body |
	body := ZnEntity 
		with: aString
		type: ZnMimeType applicationJson.

	^ZnClient new
		headerAt: 'X-Postmark-Server-Token' put: self apiKey;
		url: self apiUrl;
		entity: body.! !

!PMInterface methodsFor: 'accessing' stamp: 'FrancoisStephany 12/15/2011 18:14'!
debugStream
	^ debugStream! !

!PMInterface methodsFor: 'accessing' stamp: 'FrancoisStephany 12/15/2011 18:28'!
debugStream: aStream
	debugStream := aStream! !

!PMInterface methodsFor: 'default values' stamp: 'FrancoisStephany 12/15/2011 17:40'!
defaultApiKey
	"put the API in Test mode by default."
	^ 'POSTMARK_API_TEST'! !

!PMInterface methodsFor: 'default values' stamp: 'FrancoisStephany 12/15/2011 18:42'!
defaultApiUrl
	^'http://api.postmarkapp.com/email'! !

!PMInterface methodsFor: 'testing' stamp: 'FrancoisStephany 12/15/2011 18:14'!
developmentMode
	^debugStream isNil not.! !

!PMInterface methodsFor: 'private' stamp: 'FrancoisStephany 12/15/2011 18:55'!
handleError: aZnResponse
	aZnResponse isAuthenticationRequired
		ifTrue: [ ^PMException signal: 'Incorrect API Key.' ] .
	aZnResponse status =  422
		ifTrue: [ ^self processIncorrectFields: aZnResponse contents].
		
	PMException signal: 'Unkown error: ' , aZnResponse status printString.! !

!PMInterface methodsFor: 'private' stamp: 'FrancoisStephany 12/15/2011 18:54'!
handleResponse: aZnResponse
	aZnResponse isSuccess ifTrue: [ ^true ].
	
	self handleError: aZnResponse.
	^false! !

!PMInterface methodsFor: 'private' stamp: 'FrancoisStephany 12/15/2011 18:53'!
processIncorrectFields: aString
	| json |
	json := Json readFrom: aString readStream.
	PMException signal: 'Error : ' , (json at: 'ErrorCode') printString , ' - ' , (json at: 'Message').! !

!PMInterface methodsFor: 'public API' stamp: 'FrancoisStephany 12/26/2011 16:35'!
send: email
	"throws PMException if something goes wrong. returns true (success)/false (failure)"
	| client json |
	self developmentMode
		ifTrue: [ ^self simulateSendOf: email ].
	json := GRPharoUtf8Codec new encode: (Json render: email).
	client := self buildZnClientForContent: json.
	client post.
	
	^self handleResponse: client response.! !

!PMInterface methodsFor: 'private' stamp: 'FrancoisStephany 12/20/2011 15:52'!
simulateSendOf: aPMEmail
	aPMEmail jsonWriteOn: self debugStream.
	self debugStream cr.
	self debugStream flush.! !

JsonObject subclass: #PMEmail
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PostMark-Core'!

!PMEmail commentStamp: 'FrancoisStephany 12/20/2011 15:39' prior: 0!
See PMInterface comment for an overview.



I act like a Dictionary. According to PostMark documentation, I accept the following values:

{
  "From" : "sender@example.com",
  "To" : "receiver@example.com,receiver2@example.com",
  "Cc" : "copied@example.com",
  "Bcc": "blank-copied@example.com",
  "Subject" : "Test",
  "Tag" : "Invitation",
  "HtmlBody" : "<b>Hello</b>",
  "TextBody" : "Hello",
  "ReplyTo" : "reply@example.com",
  "Headers" : [{ "Name" : "CUSTOM-HEADER", "Value" : "value" }]
}!

!PMEmail methodsFor: 'attachments' stamp: 'FrancoisStephany 1/11/2012 00:09'!
addAttachment: name content: contentString contentType: aContentType
	| attachment encodedStream |
	encodedStream := Base64MimeConverter mimeEncode: contentString readStream.
	
	attachment := JsonObject new.
	attachment at: 'Name' put: name.
	attachment at: 'ContentType' put: aContentType asString.
	attachment at: 'Content' put: encodedStream contents.
	
	(self at: #Attachments) ifNil: [ self at: #Attachments put: OrderedCollection new ].
	(self at: #Attachments) add: attachment.! !

!PMEmail methodsFor: 'attachments' stamp: 'FrancoisStephany 1/11/2012 00:22'!
attachFile: aFSReference
	| mimeType |
	mimeType := MIMEType forFileNameReturnMimeTypesOrDefault: aFSReference basename.
	^self addAttachment: aFSReference basename content: aFSReference readStream contents contentType: mimeType displayString.! !
